@@ -23,6 +23,8 @@
  *  Boston, MA 02110-1301 USA
  */
 
+#include <gst/vaapi/gstvaapifilter.h>
+
 #include "gstvaapipostprocutil.h"
 #include "gstvaapipluginutil.h"
 
@@ -177,6 +179,24 @@ _fixate_frame_size (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
     from_w = GST_VIDEO_INFO_WIDTH (vinfo);
     from_h = GST_VIDEO_INFO_HEIGHT (vinfo);
 
+    if (postproc->has_vpp) {
+      /* adjust for crop settings */
+      from_w -= postproc->crop_left + postproc->crop_right;
+      from_h -= postproc->crop_top + postproc->crop_bottom;
+
+      /* compensate for rotation if needed */
+      switch (gst_vaapi_filter_get_video_direction (postproc->filter)) {
+        case GST_VIDEO_ORIENTATION_90R:
+        case GST_VIDEO_ORIENTATION_90L:
+        case GST_VIDEO_ORIENTATION_UL_LR:
+        case GST_VIDEO_ORIENTATION_UR_LL:
+          G_PRIMITIVE_SWAP (gint, from_w, from_h);
+          G_PRIMITIVE_SWAP (gint, from_par_n, from_par_d);
+        default:
+          break;
+      }
+    }
+
     gst_structure_get_int (outs, "width", &w);
     gst_structure_get_int (outs, "height", &h);
 
@@ -577,8 +597,8 @@ _set_multiview_mode (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
 }
 
 static gboolean
-_set_colorimetry (GstVaapiPostproc * postproc, GstVideoFormat format,
-    GstStructure * outs)
+_set_colorimetry (GstVaapiPostproc * postproc, GstVideoInfo * sinkinfo,
+    GstVideoFormat format, GstStructure * outs)
 {
   GstVideoInfo vinfo;
   GstVideoColorimetry colorimetry;
@@ -589,13 +609,20 @@ _set_colorimetry (GstVaapiPostproc * postproc, GstVideoFormat format,
       || !gst_structure_get_int (outs, "height", &height))
     return FALSE;
 
-  gst_video_info_set_format (&vinfo, format, width, height);
+  /* Use the sink resolution and the src format to correctly determine the
+   * default src colorimetry. */
+  gst_video_info_set_format (&vinfo, format, GST_VIDEO_INFO_WIDTH (sinkinfo),
+      GST_VIDEO_INFO_HEIGHT (sinkinfo));
 
   if (GST_VIDEO_INFO_CHROMA_SITE (&vinfo) != GST_VIDEO_CHROMA_SITE_UNKNOWN) {
     gst_structure_set (outs, "chroma-site", G_TYPE_STRING,
         gst_video_chroma_to_string (GST_VIDEO_INFO_CHROMA_SITE (&vinfo)), NULL);
   }
 
+  /* if outs structure already specifies colorimetry, use it */
+  if (gst_structure_has_field (outs, "colorimetry"))
+    return TRUE;
+
   /* make sure we set the RGB matrix for RGB formats */
   colorimetry = GST_VIDEO_INFO_COLORIMETRY (&vinfo);
   if (GST_VIDEO_FORMAT_INFO_IS_RGB (vinfo.finfo) &&
@@ -697,8 +724,8 @@ _get_preferred_caps (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
     goto fixate_failed;
   _set_multiview_mode (postproc, vinfo, structure);
 
-  if (f == GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY)
-    _set_colorimetry (postproc, format, structure);
+  if (!_set_colorimetry (postproc, vinfo, format, structure))
+    goto fixate_failed;
 
   if (!_set_interlace_mode (postproc, vinfo, structure))
     goto interlace_mode_failed;
@@ -715,7 +742,7 @@ _get_preferred_caps (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
     postproc->format = format;
   }
 
-  return outcaps;
+  return gst_caps_fixate (outcaps);
 
   /* ERRORS */
 fixate_failed:
